==> truffle-runtime-23.1.1-sources.jar: com/oracle/truffle/runtime/OptimizedFastThreadLocal.java <==
/*
 * Copyright (c) 2021, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.runtime;

import static com.oracle.truffle.api.CompilerAsserts.partialEvaluationConstant;
import static com.oracle.truffle.api.CompilerDirectives.inCompiledCode;
import static com.oracle.truffle.runtime.OptimizedCallTarget.castArrayFixedLength;
import static com.oracle.truffle.runtime.OptimizedCallTarget.unsafeCast;

import com.oracle.truffle.api.CompilerDirectives;
import com.oracle.truffle.api.impl.AbstractFastThreadLocal;

public abstract class OptimizedFastThreadLocal extends AbstractFastThreadLocal {

    protected OptimizedFastThreadLocal() {
    }

    @Override
    public abstract void set(Object[] data);

    @Override
    public abstract Object[] get();

    @Override
    @SuppressWarnings("unchecked")
    public final <C> C fastGet(int index, Class<C> castType, boolean invalidateOnNull) {
        Object[] data;
        if (inCompiledCode()) {
            partialEvaluationConstant(index);
            partialEvaluationConstant(castType);
            Object[] array = get();
            if (array == null) {
                if (invalidateOnNull) {
                    CompilerDirectives.transferToInterpreterAndInvalidate();
                }
                return null;
            }
            Object v = castArrayFixedLength(array, index + 1)[index];
            C result = unsafeCast(v, castType, true, false, true);
            return result;
        } else {
            data = get();
            if (data == null) {
                return null;
            }
            C result = (C) data[index];
            assert castType == null || result == null || result.getClass() == castType : "Invalid exact type returned";
            return result;
        }
    }

}

==> truffle-runtime-24.2.0-sources.jar: com/oracle/truffle/runtime/OptimizedFastThreadLocal.java <==
/*
 * Copyright (c) 2021, 2023, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * The Universal Permissive License (UPL), Version 1.0
 *
 * Subject to the condition set forth below, permission is hereby granted to any
 * person obtaining a copy of this software, associated documentation and/or
 * data (collectively the "Software"), free of charge and under any and all
 * copyright rights in the Software, and any and all patent rights owned or
 * freely licensable by each licensor hereunder covering either (i) the
 * unmodified Software as contributed to or provided by such licensor, or (ii)
 * the Larger Works (as defined below), to deal in both
 *
 * (a) the Software, and
 *
 * (b) any piece of software and/or hardware listed in the lrgrwrks.txt file if
 * one is included with the Software each a "Larger Work" to which the Software
 * is contributed by such licensors),
 *
 * without restriction, including without limitation the rights to copy, create
 * derivative works of, display, perform, and distribute the Software and make,
 * use, sell, offer for sale, import, export, have made, and have sold the
 * Software and the Larger Work(s), and to sublicense the foregoing rights on
 * either these or other terms.
 *
 * This license is subject to the following condition:
 *
 * The above copyright notice and either this complete permission notice or at a
 * minimum a reference to the UPL must be included in all copies or substantial
 * portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package com.oracle.truffle.runtime;

import static com.oracle.truffle.api.CompilerAsserts.partialEvaluationConstant;
import static com.oracle.truffle.api.CompilerDirectives.inCompiledCode;
import static com.oracle.truffle.runtime.OptimizedCallTarget.castArrayFixedLength;
import static com.oracle.truffle.runtime.OptimizedCallTarget.unsafeCast;

import com.oracle.truffle.api.CompilerDirectives;
import com.oracle.truffle.api.impl.AbstractFastThreadLocal;

public abstract class OptimizedFastThreadLocal extends AbstractFastThreadLocal {

    protected OptimizedFastThreadLocal() {
    }

    @Override
    public abstract void set(Object[] data);

    @Override
    public abstract Object[] get();

    @Override
    @SuppressWarnings("unchecked")
    public final <C> C fastGet(int index, Class<C> castType, boolean invalidateOnNull, boolean nonNullResult) {
        Object[] data;
        if (inCompiledCode()) {
            partialEvaluationConstant(index);
            partialEvaluationConstant(castType);
            Object[] array = get();
            if (array == null) {
                if (invalidateOnNull) {
                    CompilerDirectives.transferToInterpreterAndInvalidate();
                }
                return null;
            }
            Object v = castArrayFixedLength(array, index + 1)[index];
            C result = unsafeCast(v, castType, true, nonNullResult, true);
            return result;
        } else {
            data = get();
            if (data == null) {
                return null;
            }
            C result = (C) data[index];
            assert castType == null || result == null || result.getClass() == castType : "Invalid exact type returned";
            return result;
        }
    }

}
